@@ -2,13 +2,20 @@ from __future__ import annotations
 
 import logging
 import time
-import traceback
 import typing
 
 import aiohttp
 
+from iaqualink.const import (
+    AQUALINK_COMMAND_SET_AUX,
+    AQUALINK_COMMAND_SET_LIGHT,
+    AQUALINK_COMMAND_SET_TEMPS,
+)
 from iaqualink.device import AqualinkDevice
-from iaqualink.exception import AqualinkSystemOfflineException
+from iaqualink.exception import (
+    AqualinkServiceException,
+    AqualinkSystemOfflineException,
+)
 from iaqualink.typing import Payload
 
 if typing.TYPE_CHECKING:
@@ -20,7 +27,7 @@ MIN_SECS_TO_REFRESH = 15
 LOGGER = logging.getLogger("iaqualink")
 
 
-class AqualinkSystem(object):
+class AqualinkSystem:
     def __init__(self, aqualink: AqualinkClient, data: Payload):
         self.aqualink = aqualink
         self.data = data
@@ -28,7 +35,9 @@ class AqualinkSystem(object):
         self.has_spa = None
         self.temp_unit = None
         self.last_refresh = 0
-        self.last_run_success = None
+
+        # Semantics here are somewhat odd.
+        # True/False are obvious, None means "unknown".
         self.online = None
 
     def __repr__(self) -> str:
@@ -74,21 +83,19 @@ class AqualinkSystem(object):
         try:
             r1 = await self.aqualink.send_home_screen_request(self.serial)
             r2 = await self.aqualink.send_devices_screen_request(self.serial)
+        except AqualinkServiceException:
+            self.online = None
+            raise
+
+        try:
             await self._parse_home_response(r1)
             await self._parse_devices_response(r2)
         except AqualinkSystemOfflineException:
-            self.last_run_success = True
             self.online = False
-        except Exception as e:  # pylint: disable=W0703
-            self.last_run_success = False
-            self.online = None
-            LOGGER.error(f"Unhandled exception: {e}")
-            for line in traceback.format_exc().split("\n"):
-                LOGGER.error(line)
-        else:
-            self.last_run_success = True
-            self.online = True
-            self.last_refresh = int(time.time())
+            raise
+
+        self.online = True
+        self.last_refresh = int(time.time())
 
     async def _parse_home_response(
         self, response: aiohttp.ClientResponse
@@ -153,23 +160,28 @@ class AqualinkSystem(object):
                 self.devices[k] = AqualinkDevice.from_data(self, v)
 
     async def set_pump(self, command: str) -> None:
-        r = await self.aqualink.set_pump(self.serial, command)
+        r = await self.aqualink._send_session_request(self.serial, command)
         await self._parse_home_response(r)
 
     async def set_heater(self, command: str) -> None:
-        r = await self.aqualink.set_heater(self.serial, command)
+        r = await self.aqualink._send_session_request(self.serial, command)
         await self._parse_home_response(r)
 
     async def set_temps(self, temps: Payload) -> None:
-        r = await self.aqualink.set_temps(self.serial, temps)
+        r = await self.aqualink._send_session_request(
+            self.serial, AQUALINK_COMMAND_SET_TEMPS, temps
+        )
         await self._parse_home_response(r)
 
     async def set_aux(self, aux: str) -> None:
-        r = await self.aqualink.set_aux(self.serial, aux)
+        aux = AQUALINK_COMMAND_SET_AUX + "_" + aux.replace("aux_", "")
+        r = await self.aqualink._send_session_request(self.serial, aux)
         await self._parse_devices_response(r)
 
     async def set_light(self, data: Payload) -> None:
-        r = await self.aqualink.set_light(self.serial, data)
+        r = await self.aqualink._send_session_request(
+            self.serial, AQUALINK_COMMAND_SET_LIGHT, data
+        )
         await self._parse_devices_response(r)
 
 
